@@ -58,7 +58,7 @@ static int get_tag(AVIOContext *pb, uint32_t * tag)
 {
     int size;
 
-    if (url_feof(pb))
+    if (avio_feof(pb))
         return AVERROR(EIO);
 
     *tag = avio_rl32(pb);
